/* check internal consistency of dpf_base */
#include <dpf-internal.h>
#include <string.h>

static int ck_pid_table[DPF_MAXFILT];

static int seen_pid(int pid) {
        if(pid) {
                if(!ck_pid_table[pid])
                        ck_pid_table[pid] = 1;
                else {
                        printf("***\npid %d has been seen twice\n\n", pid);
                        dpf_output();
                        return 1;
                }
        }
        return 0;
}

static void cktrie(Atom parent, Atom a, int refcnt) {
        int n, htn;
        Ht ht;
        Atom or, prev;

	if(!a)
		return;

        for(prev = 0, n = 0; a; prev = a, a = a->sibs.le_next) {
                if(a->parent != parent) {
			printf("\n********\nbogus parent!\n\n");
			dpf_output();
 			fatal(bogus parent);
		}
                n += a->refcnt;

                demand(!seen_pid(a->pid), already seen pid);

                if(!(ht = a->ht)) {
                        cktrie(a, a->kids.lh_first, a->refcnt - !!a->pid);
                        continue;
                }
		demand(!a->pid, ht should not have a pid);

		for(htn = 0, or = a->kids.lh_first; or; or = or->sibs.le_next) {
                	demand(or->parent == a, bogus parent);
                	htn += or->refcnt;
                	cktrie(or, or->kids.lh_first, or->refcnt - !!or->pid);
		}
	
                if(htn != a->refcnt) {
                        printf("****\na->refcnt == %d, htn = %d\n\n",
                                a->refcnt, htn);
                        dpf_output();
                        fatal(bogus refcnt);
                }
        }

        if(n == refcnt)
                return;

        printf("****\nrefcnt == %d, n = %d\n\n", refcnt, n);
        dpf_output();
        fatal(bogus refcnt);
}

void dpf_check(void) {
#ifndef NDEBUG
        int i, refcnt;
	Atom base;

        if(!(base = dpf_base->kids.lh_first))
                return;

        memset(ck_pid_table, 0, sizeof ck_pid_table);

        for(refcnt = i = 0; i < DPF_MAXFILT; i++)
                if(dpf_active[i])
                        refcnt++;

        cktrie(dpf_base, base, refcnt);
#endif
}
