#ifndef __DEMAND__
#define __DEMAND__
#include <stdio.h>

#ifdef NDEBUG
#       define demand(bool, msg)        do { } while(0)
#       define assert(bool)             do { } while(0)
#       define fatal(msg)               do { } while(0)
#	define debug_stmt(s)		do { } while(0)
#else /* NDEBUG */
#	define debug_stmt(s)		do { s; } while(0)
#       if defined(__GNUC__) || defined(__ANSI__)
#               define assert(bool) do {                                \
                        if(bool)                                        \
                                (void)0;                                \
                        else                                            \
                                __assert(#bool, __FILE__, __LINE__);    \
                } while(0)

#               define demand(bool, msg) do {                           \
                        if(bool)                                        \
                                (void)0;                                \
                        else                                            \
                                __demand(#bool, #msg, __FILE__, __LINE__);\
                } while(0)

#               define fatal(msg) __fatal(__FILE__, __LINE__, #msg)

#       else /* __GNUC__ */

#               define assert(bool) do {                                \
                        if(bool)                                        \
                                (void)0;                                \
                        else                                            \
                                __assert("bool", __FILE__, __LINE__);   \
                } while(0)

#               define demand(bool, msg) do {                           \
                        if(bool)                                        \
                                (void)0;                                \
                        else                                            \
                                __demand("bool", "msg", __FILE__, __LINE__);\
                } while(0)

#               define fatal(msg) __fatal(__FILE__, __LINE__, "msg")
#       endif   /* __GNUC__ */

#define __demand(_bool, _msg, _file, line) do {                         \
        printf("file %s, line %d: %s\n",_file,line,_bool);              \
        printf("%s\n",_msg);                                            \
        exit(1);                                                        \
} while(0)

#define __assert(_bool, _file, line) do {                               \
        printf("file %s, line %d: %s\n",_file,line,_bool);              \
        exit(1);                                                        \
} while(0)

#define __fatal(_file, _line, _msg) do {                                \
        printf("PANIC: %s\n",_msg);                                     \
        printf("(file %s, line %d)\n",_file,_line);                     \
        exit(1);                                                        \
} while(0)

void exit(int);
void panic (char *str);
#endif /* NDEBUG */
        
/* Compile-time assertion used in function. */
#define AssertNow(x) switch(1) { case (x): case 0: }

/* Compile time assertion used at global scope. */
#define _assert_now2(y) _ ##  y
#define _assert_now1(y) _assert_now2(y)
#define AssertNowF(x) \
  static inline void _assert_now1(__LINE__)() { \
                /* used to shut up warnings */                  \
                void (*fp)(int) = _assert_now1(__LINE__);       \
                fp(1);                                          \
                switch(1) case (x): case 0: ;                   \
  }

#endif
