#include <dpf-internal.h>
#include <string.h>
#include <stdlib.h>
#include "stopwatch.h"

/* simple filter before trying the harder ones. */
int main(int argc, char *argv[]) { 
	struct dpf_ir ir, ir2;
	uint8 msg[128], msg2[128], junk[128];
	int pid, pid2, i, n;

	n = (argc == 2) ? atoi(argv[1]) : 1000;

	if(!n)
		dpf_verbose(1);

	/* simple checker */
	dpf_init(&ir);
	dpf_eq16(&ir, 12, 0x1234);
	dpf_eq8(&ir, 8, 0x12);
	dpf_shifti(&ir, 16);
	dpf_eq32(&ir, 0, 0x12345678);

	*(uint8 *) ((uint8 *)msg + 8) = 0x12;
	*(uint16 *) ((uint8 *)msg + 12) = 0x1234;
	*(uint32 *) ((uint8 *)msg + 16) = 0x12345678;

	if((pid = dpf_insert(&ir)) < 0)
		fatal(Insertion failed);

	dpf_output();

	if(pid != dpf_iptr(msg, sizeof msg))
		fatal(Did not recognize the message.);
	if(pid != dpf_iptr(msg, sizeof msg))
		fatal(Did not recognize the message.);
	if(pid == dpf_iptr(junk, sizeof junk))
		fatal(Should not recognize the message.);

	/* should not let us insert again. */
	if(dpf_insert(&ir) >= 0)
		fatal(Insertion should have failed);

	if(pid != dpf_iptr(msg, sizeof msg))
		fatal(Did not recognize the message.);
	if(pid == dpf_iptr(junk, sizeof junk))
		fatal(Should not recognize the message.);

	if(dpf_insert(&ir) >= 0)
		fatal(Insertion should have failed);
	dpf_output();

	if(pid != dpf_iptr(msg, sizeof msg))
		fatal(Did not recognize the message.);
	if(pid == dpf_iptr(junk, sizeof junk))
		fatal(Should not recognize the message.);

	/* hash table */
	*(uint8 *) ((uint8 *)msg2 + 8) = 0x13;
	*(uint16 *) ((uint8 *)msg2 + 12) = 0x1234;
	*(uint32 *) ((uint8 *)msg2 + 16) = 0x12345677;
	dpf_init(&ir2);
	dpf_eq16(&ir2, 12, 0x1234);
	dpf_eq8(&ir2, 8, 0x13);
	dpf_shifti(&ir2, 16);
	dpf_eq32(&ir2, 0, 0x12345677);
	dpf_output();

	if((pid2 = dpf_insert(&ir2)) < 0)
		fatal(Insertion failed);
	if(pid != dpf_iptr(msg, sizeof msg))
		fatal(Did not recognize the message.);
	if(pid2 != dpf_iptr(msg2, sizeof msg2))
		fatal(Did not recognize the message.);
	if(dpf_delete(pid2) < 0)
		fatal(Insertion failed);

	/* test for longest match */
	dpf_init(&ir2);
	dpf_eq16(&ir2, 12, 0x1234);
	dpf_eq8(&ir2, 8, 0x12);
	if((pid2 = dpf_insert(&ir2)) < 0)
		fatal(Insertion failed);
	dpf_output();
	
	if(pid != dpf_iptr(msg, sizeof msg))
		fatal(Did not recognize the message.);

	if(dpf_delete(pid) < 0)
		fatal(Insertion failed);
	dpf_output();

	if(pid2 != dpf_iptr(msg, sizeof msg))
		fatal(Did not recognize the message.);

	if((pid = dpf_insert(&ir)) < 0)
		fatal(Insertion failed);

	if(pid != dpf_iptr(msg, sizeof msg))
		fatal(Did not recognize the message.);

	if(dpf_delete(pid2) < 0)
		fatal(Insertion failed);

	if(pid != dpf_iptr(msg, sizeof msg))
		fatal(Did not recognize the message.);

	if((pid2 = dpf_insert(&ir2)) < 0)
		fatal(Insertion failed);
	
	if(dpf_delete(pid2) < 0)
		fatal(Insertion failed);

	if(pid != dpf_iptr(msg, sizeof msg))
		fatal(Did not recognize the message.);
	dpf_output();

	/***************************** 
	 *   OR test.
	 */

	dpf_init(&ir2);
	dpf_eq16(&ir2, 12, 0x5678);
	dpf_shifti(&ir2, 16);
	dpf_eq32(&ir2, 0, 0x12345678);

	*(uint8 *) ((uint8 *)msg2 + 8) = 0x12;
	*(uint16 *) ((uint8 *)msg2 + 12) = 0x5678;
	*(uint32 *) ((uint8 *)msg2 + 16) = 0x12345678;

	if((pid2 = dpf_insert(&ir2)) < 0)
		fatal(Insertion failed);

	dpf_output();
	demand(dpf_insert(&ir2)  < 0, Insertion failed);

	if(pid2 != dpf_iptr(msg2, sizeof msg2))
		fatal(Did not recognize the message.);
	if(pid != dpf_iptr(msg, sizeof msg))
		fatal(Did not recognize the message.);

	dpf_output();

	if(dpf_delete(pid2) < 0)
		fatal(Insertion failed);
	if(pid != dpf_iptr(msg, sizeof msg))
		fatal(Did not recognize the message.);

	dpf_output();

	if((pid2 = dpf_insert(&ir2)) < 0) {
		printf("errno = %d\n", pid2);
		fatal(Insertion failed);
	}


	if(pid2 != dpf_iptr(msg2, sizeof msg2))
		fatal(Did not recognize the message.);

	if(pid != dpf_iptr(msg, sizeof msg))
		fatal(Did not recognize the message.);

	if(dpf_delete(pid2) < 0)
		fatal(Insertion failed);

	dpf_output();

	/* ************************
	 * shifts
	 */
	printf("the following shifts should give 4 byte aligment\n");
	dpf_init(&ir2);
	dpf_shift8(&ir2, 0, 4);
	dpf_mshift8(&ir2, 0, ~0x3, 0);
	dpf_mshift16(&ir2, 0, ~0x1, 2);
	dpf_mshift16(&ir2, 0, 0xff, 4);

	if((pid2 = dpf_insert(&ir2)) < 0)
		fatal(Insertion failed);
	dpf_output();
	if(dpf_delete(pid2) < 0)
		fatal(Insertion failed);
	dpf_output();

	printf("the following shifts should give 2 byte aligment\n");
	dpf_init(&ir2);
	dpf_shift8(&ir2, 0, 2);
	dpf_mshift16(&ir2, 0, ~0x1, 0);
	dpf_mshift16(&ir2, 0, 0xf, 2);

	if((pid2 = dpf_insert(&ir2)) < 0)
		fatal(Insertion failed);
	dpf_output();
	if(dpf_delete(pid2) < 0)
		fatal(Insertion failed);
	dpf_output();

	printf("the following shifts should give 1 byte aligment\n");
	dpf_init(&ir2);
	dpf_shift8(&ir2, 0, 0);
	dpf_mshift16(&ir2, 0, ~0xff, 8);
	dpf_mshift16(&ir2, 0, ~0xff, 2);

	if((pid2 = dpf_insert(&ir2)) < 0)
		fatal(Insertion failed);
	dpf_output();
	if(dpf_delete(pid2) < 0)
		fatal(Insertion failed);

	dpf_output();

	/******************************
	 * Hash table test.
	 */
	dpf_init(&ir2);
	dpf_eq16(&ir2, 12, 0x1234);
	dpf_eq8(&ir2, 8, 0x12);
	dpf_shifti(&ir2, 16);
	dpf_eq32(&ir2, 0, 0x12345670);

	memset(msg2, 0, sizeof msg2);
	*(uint8 *) ((uint8 *)msg2 + 8) = 0x12;
	*(uint16 *) ((uint8 *)msg2 + 12) = 0x1234;
	*(uint32 *) ((uint8 *)msg2 + 16) = 0x12345670;

	if((pid2 = dpf_insert(&ir2)) < 0)
		fatal(Insertion failed);
	dpf_output();

	if(pid2 != dpf_iptr(msg2, sizeof msg2))
		fatal(Did not recognize the message.);

	if(pid != dpf_iptr(msg, sizeof msg))
		fatal(Did not recognize the message.);

	if(dpf_delete(pid) < 0)
		fatal(Insertion failed);

	dpf_output();

	if(pid2 != dpf_iptr(msg2, sizeof msg2))
		fatal(Did not recognize the message.);

	if(pid == dpf_iptr(msg, sizeof msg))
		fatal(Recognized the message.);

	if(dpf_delete(pid2) < 0)
		fatal(Insertion failed);

	if(dpf_iptr(msg2, sizeof msg2))
		fatal(recognized the message.);

	dpf_output();

	if((pid2 = dpf_insert(&ir2)) < 0)
		fatal(Insertion failed);

	/* test insertion and deletion */
	startwatch(0);
	for(i = 0; i < n; i++) {

		if((pid = dpf_insert(&ir)) < 0)
			fatal(Insertion failed);

		if(dpf_delete(pid) < 0)
			fatal(Deletion failed);
	}
	stopwatch(0);

	printf("Success!\n");

	return 0;
}
